<?php

namespace Drupal\pagedesigner\Plugin\pagedesigner\Handler;

use Drupal\Core\Render\Element as RenderElement;
use Drupal\pagedesigner\Entity\Element;
use Drupal\pagedesigner\Plugin\CompoundHandlerBase;

/**
 * Process entities of type "row".
 *
 * @PagedesignerHandler(
 *   id = "row",
 *   name = @Translation("Row render"),
 *   types = {
 *     "row",
 *   },
 * )
 */
class Row extends CompoundHandlerBase {

  /**
   * {@inheritdoc}
   */
  public function serialize(Element $entity, array &$result = []) {
    parent::serialize($entity, $result);
    $result['type'] = 'row';
  }

  /**
   * {@inheritdoc}
   */
  public function describe(Element $entity, array &$result = []) {
    parent::describe($entity, $result);
    $result['type'] = 'row';
  }

  /**
   * {@inheritdoc}
   */
  public function view(Element $entity, string $view_mode, array &$build = []) {
    $children = $this->collectChildren($entity, $view_mode);
    foreach ($children as $key => $child) {
      $build[$key] = $this->viewBuilder->view($child, $view_mode);
    }
    $build['#pre_render'][] = [$this->viewBuilder, 'render'];
    $this->addStyles($entity, $view_mode == 'public');
  }

  /**
   * {@inheritdoc}
   */
  public function build(Element $entity, string $view_mode, array &$build = []) {
    $pattern = $entity->field_pattern->value;
    $build = [
      '#type' => 'pattern',
      '#id' => $pattern,
      '#fields' => [],
    ] + $build;
    $children = RenderElement::children($build);
    $fields = [];
    foreach ($children as $key) {
      $child = $build[$key]['#pagedesigner_element'];
      $placeholder = $child->field_placeholder->value;
      if ($child->bundle() == 'cell') {
        // For cells, add field with reference to renderable child.
        $fields[$placeholder] = &$build[$key];
      }
      else {
        // For value types, build the child and add the result in the field.
        $sub_build = $this->elementHandler->build($child, $view_mode);
        $fields[$placeholder] = $sub_build;
      }
    }
    $build['#fields'] = $fields;
    $build['#post_render'][] = [$this->viewBuilder, 'postRender'];
    unset($build['#pre_render']);
    $build = [
      'pagedesigner_structure' => $build,
    ];
  }

}
